use rand::{distributions::Alphanumeric, thread_rng, Rng};
use sqlx::SqlitePool;

use crate::types::uuid::Uuid;

use super::devices::Device;

pub struct Session {
    pub uuid: Uuid,
    pub device_uuid: Uuid,
    pub key: String,
}

impl Session {
    pub fn new(device: &Device) -> anyhow::Result<Self> {
        let mut rng = thread_rng();
        let key: String = (0..256).map(|_| rng.sample(Alphanumeric) as char).collect();

        Ok(Self {
            uuid: uuid::Uuid::new_v4().into(),
            device_uuid: device.uuid().clone(),
            key,
        })
    }

    pub async fn create(&self, conn: &SqlitePool) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "insert into sessions(uuid, device_uuid, key)
            values(?, ?, ?)
            returning uuid as 'uuid: Uuid', device_uuid as 'device_uuid: Uuid', key",
            self.uuid,
            self.device_uuid,
            self.key
        )
        .fetch_one(conn)
        .await?)
    }

    pub async fn find_by_key(conn: &SqlitePool, key: &str) -> anyhow::Result<Option<Self>> {
        Ok(sqlx::query_as!(
            Self,
            "select uuid as 'uuid: Uuid', device_uuid as 'device_uuid: Uuid', key
            from sessions where key = ?",
            key
        )
        .fetch_optional(conn)
        .await?)
    }

    pub async fn device(&self, conn: &SqlitePool) -> anyhow::Result<Device> {
        Device::find_by_uuid(conn, &self.device_uuid).await
    }
}
